# RandomCells

This is a [Julia](https://julialang.org/) implementation of some standard models for random convex polytopes in dimensions 2 and 3. More precisely, these are certain cells of a random stationary and isotropic mosaic, i.e. a random decomposition of the plane or space into (convex) regions. 

The output type of the polytopes is `Polyhedra.polyhedron`, which means that the functionality of the Julia package [Polyhedra](https://juliapolyhedra.github.io/Polyhedra.jl) can be used in a simple way if desired.

In addition, the package also implements some standard statistics for polygons (area, perimeter, mean width, number of sides) and polyhedrons (volume, surface area, mean width, number of sides).

## Description
The cells under consideration are on the one hand the _zero cell_ of the mosaic, i.e. the cell that (almost surely) contains the origin, and on the other hand the so-called _typical cell_ of the mosaic, which can be understood as the result of a random selection among all cells of the mosaic, in which all cells have the same chance of being selected.

- The first mosaic/tessellation type is the _Poisson-Voronoi mosaic_. The starting point for its construction is a set of spatial seed-points following a homogeneous Poisson point process. According to the general construction principle of a Voronoi mosaic, each seed-point is then assigned a cell consisting of all points of 2D/3D space that are closer to the generating point than to all other generating points. Besides dimension there is one model parameter 'τ' equal to the point intensity of the underlying Poisson point process. See Schneider & Weil (2008) and Chiu et al. (2013) for the theoretical background.

- The second mosaic/tessellation type is the stationary and isotropic _Poisson line/plane tessellation_. Therein the space is divided into polygonal/polyhedral cells by (infinite) lines/planes which exhibit random orientation and location such that the entire system is both stationary and isotropic in 2D/3D space. Besides dimension there is one model parameter 'λ' equal to the mean line length per unit area/mean plane area per unit volume of the underlying Poisson line/plane tessellation. See Schneider & Weil (2008) and Chiu et al. (2013) for the theoretical background.

The implementations for the typical cell each use a direct method that does not simulate entire mosaics, see Lantuéjoul (2002) and Calka (2010).

## Usage

```julia
using RandomCells
```

### Typical cell of the Poisson-Voronoi mosaic

```julia
PVT2 = PoissonVoronoiCell(2, 1.0); # dimension = 2, intensity parameter τ = 1.0
PVT3 = PoissonVoronoiCell(3, 1.0); # dimension = 3, intensity parameter τ = 1.0
```

### Zero cell of the Poisson-Voronoi mosaic

```julia
PVZ2 = PoissonVoronoiZeroCell(2, 1.0); # dimension = 2, intensity parameter τ = 1.0
PVZ3 = PoissonVoronoiZeroCell(3, 1.0); # dimension = 3, intensity parameter τ = 1.0
```

### Typical cell of the Poisson line/plane tessellation (Poisson cell)

```julia
PHT2 = PoissonCell(2, 1.0); # dimension = 2, intensity parameter λ = 1.0
PHT3 = PoissonCell(3, 1.0); # dimension = 3, intensity parameter λ = 1.0
```

### Zero cell of the Poisson line/plane tessellation (Crofton cell)

```julia
PHZ2 = CroftonCell(2, 1.0); # dimension = 2, intensity parameter λ = 1.0
PHZ3 = CroftonCell(3, 1.0); # dimension = 3, intensity parameter λ = 1.0
```

### Various statistics
```julia
getVolume(PHT2)      # area/volume
getSurface(PHT2)     # perimeter/surface area
getMeanWidth(PHT2)   # mean width
getFacetNumber(PHT2) # number of sides/facets
```

## License
This code is free to use under the terms of the MIT license.

## References
- Calka, P. (2010). Tessellations. In: W. Kendall and I. Molchanov (eds), New Perspectives in Stochastic Geometry. Oxford University Press, pp. 145–169.
- Chiu, S. N., Stoyan, D., Kendall, W. S. and Mecke, J. (2013), Stochastic Geometry and its Applications, 3rd edn, J. Wiley & Sons, Chichester.
- Lantuéjoul, C. (2002). Geostatistical Simulation. Springer, Berlin.
- Schneider, R. and Weil, W. (2008), Stochastic and Integral Geometry, Springer, Berlin.


