function randomInitialSimplex!(u::Vector{Vector{Float64}})
    D = length(u[1])
    maxvol = 2.0
    if D == 2
        maxvol = 1.299038105676658
        # = 3*sqrt(3)/4, equals the maximum area of a triangle with vertices on the unit circle
    end
    if D == 3
        maxvol = 0.5132002392796674
        # = 8/9/sqrt(3), equals the maximum volume of a tetrahedron with vertices on the unit sphere
    end
    origin = zeros(SVec{D})
    once_again = true
    while once_again
        origin_inside_simplex = false
        while !origin_inside_simplex
            u .= [uniformDirection(D) for _ in 0:D]
            origin_inside_simplex = inSimplex(origin, u)
        end
        once_again = ( getVolumeOfSimplex(u) < maxvol * rand() )
    end
    return nothing
end

"Generate a realization of the Poisson cell with the intensity parameter λ."
function PoissonCell(dim::Integer=2, λ::Real=1.0)
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    if λ <= 0.0
        println("The intensity parameter λ must be positive.")
        return nothing
    end
    uu = [Vector{Float64}(undef, dim) for _ in 0:dim]
    randomInitialSimplex!(uu)
    normals = [SVec{dim}(u0) for u0 in uu]
    rsimplex = rand( Distributions.Exponential(0.5 / λ) )
    distances = [rsimplex for _ in 0:dim]
    rmax = getMaxNorm( asPolyhedron(normals, distances) )
    rstep = 10.0 / λ
    rlower = rsimplex - rstep
    rupper = rsimplex

    stop = false
    while !stop
        rlower += rstep
        if rupper + rstep > rmax
            rstep = rmax - rupper
            stop = true
        end
        rupper += rstep

        nnew = rand(Distributions.Poisson(2 * rstep * λ))
        if nnew > 0
            append!(normals, [uniformDirection(dim) for _ in 1:nnew])
            append!(distances, rlower .+ rstep .* rand(nnew))
        end
        rmax = getMaxNorm( asPolyhedron(normals, distances) )
        if rmax < rupper
            stop = true
        end
    end
    keep = getActiveConstraints(normals, distances)
    return asPolyhedron(normals[keep], distances[keep])
end

"Generate a realization of the Crofton cell with the intensity parameter λ."
function CroftonCell(dim::Integer=2, λ::Real=1.0)
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    if λ <= 0.0
        println("The intensity parameter λ must be positive.")
        return nothing
    end
    origin = zeros(SVec{dim})
    normals = SVec{dim}[]
    distances = Float64[]
    rstep = 10.0 / λ
    rlower = -rstep
    rupper = 0.0

    bounded = false
    stop = false
    while !stop
        rlower += rstep
        rupper += rstep

        nnew = rand(Distributions.Poisson(2 * rstep * λ))
        if nnew > 0
            append!(normals, [uniformDirection(dim) for _ in 1:nnew])
            append!(distances, rlower .+ rstep .* rand(nnew))
        end
        if !bounded
            bounded = length(distances) <= dim ? false : inPolytope(origin, polarDual(normals, distances))
        end
        if bounded
            rmax = getMaxNorm( asPolyhedron(normals, distances) )
            if rmax < rupper
                stop = true
            end
        end
    end
    keep = getActiveConstraints(normals, distances)
    return asPolyhedron(normals[keep], distances[keep])
end

"Generate a realization of the typical cell of a Poisson-Voronoi tessellation with the intensity parameter τ."
function PoissonVoronoiCell(dim::Integer=2, τ::Real=1.0)
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    if τ <= 0.0
        println("The intensity parameter τ must be positive.")
        return nothing
    end
    if dim == 2
        taubd = pi * τ
    end
    if dim == 3
        taubd = 4.0 * pi / 3.0 * τ
    end
    origin = zeros(SVec{dim})
    normals = [uniformDirection(dim) for _ in 0:dim]
    distances = 0.5 .* root.(cumsum( rand( Distributions.Exponential(1.0 / taubd), dim+1 ) ), dim)
    q = (2.0 * last(distances))^dim
    # If the origin is in (the interior of) the polar dual then the polyhedron is bounded.
    while !inPolytope(origin, polarDual(normals, distances))
        push!(normals, uniformDirection(dim))
        q += rand( Distributions.Exponential(1.0 / taubd) )
        push!(distances, 0.5 * root(q, dim))
    end
    # Check whether or not a further point/halfspace could influence the current polyhedron.
    q += rand( Distributions.Exponential(1.0 / taubd) )
    pnew = 0.5 * root(q, dim)
    while pnew < getMaxNorm( asPolyhedron(normals, distances) )
        push!(normals, uniformDirection(dim))
        push!(distances, pnew)
        q += rand( Distributions.Exponential(1.0 / taubd) )
        pnew = 0.5 * root(q, dim)
    end
    keep = getActiveConstraints(normals, distances)
    return asPolyhedron(normals[keep], distances[keep])
end

"Generate a realization of the zero cell of a Poisson-Voronoi tessellation with the intensity parameter τ."
function PoissonVoronoiZeroCell(dim::Integer=2, τ::Real=1.0)
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    if τ <= 0.0
        println("The intensity parameter τ must be positive.")
        return nothing
    end
    if dim == 2
        taubd = pi * τ
    end
    if dim == 3
        taubd = 4.0 * pi / 3.0 * τ
    end
    origin = zeros(SVec{dim})
    y = zeros(StaticArrays.MVector{dim, Float64})
    normals = SVec{dim}[]
    distances = Float64[]
    q = rand( Distributions.Exponential(1.0 / taubd) )
    x = root(q, dim) .* uniformDirection(dim)
    for i in 0:dim
        q += rand( Distributions.Exponential(1.0 / taubd) )
        y .= root(q, dim) .* uniformDirection(dim)
        push!(normals, LinearAlgebra.normalize(y .- x))
        push!(distances, LinearAlgebra.dot(last(normals), 0.5 .* (y .+ x)))
    end
    while !inPolytope(origin, polarDual(normals, distances))
        q += rand( Distributions.Exponential(1.0 / taubd) )
        y .= root(q, dim) .* uniformDirection(dim)
        push!(normals, LinearAlgebra.normalize(y .- x))
        push!(distances, LinearAlgebra.dot(last(normals), 0.5 .* (y .+ x)))
    end
    q += rand( Distributions.Exponential(1.0 / taubd) )
    y .= root(q, dim) .* uniformDirection(dim)
    pnew = LinearAlgebra.dot(LinearAlgebra.normalize(y .- x), 0.5 .* (y .+ x))
    while pnew < getMaxNorm( asPolyhedron(normals, distances) )
        push!(normals, LinearAlgebra.normalize(y .- x))
        push!(distances, pnew)
        q += rand( Distributions.Exponential(1.0 / taubd) )
        y .= root(q, dim) .* uniformDirection(dim)
        pnew = LinearAlgebra.dot(LinearAlgebra.normalize(y .- x), 0.5 .* (y .+ x))
    end
    keep = getActiveConstraints(normals, distances)
    return asPolyhedron(normals[keep], distances[keep])
end