"Compute the mean vector."
function meanVector(x::Vector{V}) where {V<:AbstractVector{T}} where {T<:Real}
    m = x[1]
    for i in 2:length(x)
        m = m .+ x[i]
    end
    return m ./ length(x)
end

"Compute the mean vector of the vertices."
function meanVector(poly::P) where {P<:Polyhedra.Polyhedron}
    points = Polyhedra.points(poly) 
    m = zeros(StaticArrays.MVector{Polyhedra.fulldim(poly), Float64})
    for x in points
        m .+= x
    end
    return eltype(points)(m ./ length(points))
end

function getTriangleArea(p1::SVec{3}, p2::SVec{3})
    return 0.5 * LinearAlgebra.norm( LinearAlgebra.cross(p1, p2) )
end

function getVertexOrder(uu::Vector{SVec{3}}, n::SVec{3})
    return sortperm([atan( LinearAlgebra.dot( LinearAlgebra.cross(n, uu[1]), u ), LinearAlgebra.dot(uu[1], u) ) for u in uu])
end

function getFacetContent(u::SVec{2}, points::Vector{SVec{2}})
    return LinearAlgebra.norm(points[1] .- points[2])
end

function getFacetContent(u::SVec{3}, points::Vector{SVec{3}})
    m = meanVector(points)
    order = getVertexOrder([LinearAlgebra.normalize(p .- m) for p in points], LinearAlgebra.normalize(u))
    area = getTriangleArea(points[order[1]] .- m, points[order[2]] .- m)
    for i in 2:(length(order)-1)
        area += getTriangleArea(points[order[i]] .- m, points[order[i+1]] .- m)
    end
    area += getTriangleArea(points[last(order)] .- m, points[order[1]] .- m)
    return area
end

"Compute the volume (3D) or area (2D) of a convex polytope."
function getVolume(poly::P) where {P<:Polyhedra.Polyhedron}
    dim = Polyhedra.fulldim(poly)
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    volume = 0.0
    m = meanVector(poly)
    for i in eachindex(Polyhedra.halfspaces(poly))
        h = get(poly, i)
        volume += ( h.β - LinearAlgebra.dot(m, h.a) ) * getFacetContent(h.a, Polyhedra.incidentpoints(poly, i))
    end
    return volume / length(m)
end

"Compute the surface area (3D) or boundary length (2D) of a convex polytope."
function getSurface(poly::P) where {P<:Polyhedra.Polyhedron}
    dim = Polyhedra.fulldim(poly)
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    surface = 0.0
    for i in eachindex(Polyhedra.halfspaces(poly))
        h = get(poly, i)
        surface += getFacetContent(h.a, Polyhedra.incidentpoints(poly, i))
    end
    return surface
end

"Compute the volume and surface area (3D) or area and boundary length (2D) of a convex polytope."
function getVolumeAndSurface(poly::P) where {P<:Polyhedra.Polyhedron}
    dim = Polyhedra.fulldim(poly)
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    volume = 0.0
    surface = 0.0
    m = meanVector(poly)
    for i in eachindex(Polyhedra.halfspaces(poly))
        h = get(poly, i)
        facetcontent = getFacetContent(h.a, Polyhedra.incidentpoints(poly, i))
        volume += ( h.β - LinearAlgebra.dot(m, h.a) ) * facetcontent
        surface += facetcontent
    end
    return volume / length(m), surface
end

"Compute the mean width of a convex polytope."
function getMeanWidth(poly::P) where {P<:Polyhedra.Polyhedron}
    dim = Polyhedra.fulldim(poly)
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    if dim == 2
        return getSurface(poly) / pi
    end
    value = 0.0
    hh = [get(poly, i) for i in eachindex(Polyhedra.halfspaces(poly))]
    pp = [Polyhedra.incidentpoints(poly, i) for i in eachindex(Polyhedra.halfspaces(poly))]
    n = length(hh)
    for i in 1:(n-1)
        for j in (i+1):n
            p = intersect(pp[i], pp[j])
            if length(p) == 2
                value += LinearAlgebra.norm(p[1] .- p[2]) * acos( min(1.0, LinearAlgebra.dot(hh[i].a,  hh[j].a) ) )
            end
        end
    end
    return value * 0.25 / pi
end

"Get the number of facets of a convex polytope."
function getFacetNumber(poly::P) where {P<:Polyhedra.Polyhedron}
    return Polyhedra.nhalfspaces(poly)
end

"Collect certain statistics of a convex polytope."
function getStats(poly::P) where {P<:Polyhedra.Polyhedron}
    dim = Polyhedra.fulldim(poly)
    if dim == 2
        return [getVolume(poly), getSurface(poly), getFacetNumber(poly)]
    end
    if dim == 3
        return [getVolume(poly), getSurface(poly), getMeanWidth(poly), getFacetNumber(poly)]
    end
    println("This method is only implemented for dimensions 2 and 3.")
    return nothing
end

"Collect statistics of 'n' Poisson cells."
function getStatsPoissonCell(n::Integer=1; dim::Integer=2, λ::Real=1.0)
    if n < 1
        return nothing
    end
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    if λ <= 0.0
        println("The intensity parameter λ must be positive.")
        return nothing
    end
    return n == 1 ? getStats( PoissonCell(dim, λ) ) : [getStats( PoissonCell(dim, λ) ) for _ in 1:n]
end

"Collect statistics of 'n' Crofton cells."
function getStatsCroftonCell(n::Integer=1; dim::Integer=2, λ::Real=1.0)
    if n < 1
        return nothing
    end
    if (dim < 2) | (dim > 3)
        println("This method is only implemented for dimensions 2 and 3.")
        return nothing
    end
    if λ <= 0.0
        println("The intensity parameter λ must be positive.")
        return nothing
    end
    return n == 1 ? getStats( CroftonCell(dim, λ) ) : [getStats( CroftonCell(dim, λ) ) for _ in 1:n]
end
