"Draw a random direction uniformly from the unit sphere."
function uniformDirection(dim::Integer)
    @fastmath phi = 2.0 * pi * rand()
    if dim == 2
        @fastmath return SVec{2}(cos(phi), sin(phi))
    end
    if dim == 3
        @fastmath theta = acos(2.0 * rand() - 1.0)
        @fastmath sinTheta = sin(theta)
        @fastmath cosTheta = cos(theta)
        @fastmath return SVec{3}(cos(phi)*sinTheta, sin(phi)*sinTheta, cosTheta)
    end
    return LinearAlgebra.normalize(SVec{dim}(Base.randn(dim)))
end

"Check whether the point 'x' is inside a D-simplex given by the vertices 'v'."
function inSimplex(x::V, v::Vector{W}) where {V<:AbstractVector{S}, W<:AbstractVector{T}} where {S<:Real, T<:Real}
    D = length(x)
    N = D+1
    A = StaticArrays.MMatrix{D, D, Float64}([ v[i][k]-v[1][k] for k = 1:D, i = 2:N ])
    b = x .- v[1]
    s = A \ b
    return all(s .> 0.0) & (sum(s) < 1.0)
end

"Check whether the point 'x' is inside a polytope given by the vertices 'v'."
function inPolytope(x::V, v::Vector{W}) where {V<:AbstractVector{S}, W<:AbstractVector{T}} where {S<:Real, T<:Real}
    triangulation = getDelaunay(v)
    for c in eachcol(triangulation)
        if inSimplex(x, v[c])
            return true
        end
    end
    return false
end

"Compute the volume of a D-simplex given by the vertices 'v'."
function getVolumeOfSimplex(v::Vector{V}) where {V<:AbstractVector{T}} where {T<:Real}
    D = length(v[1])
    return abs( LinearAlgebra.det( stack(v[1:D]) .- v[D+1] ) ) / factorial(D)
end

"Compute the squared Euclidean norm of a vector."
function sqrNorm(x::V) where {V<:AbstractVector{T}} where {T<:Real}
    return LinearAlgebra.dot(x, x)
end

"Compute the distance of the farthest vertex from the origin."
function getMaxNorm(poly::P) where {P<:Polyhedra.Polyhedron}
    m = 0.0
    for x in Polyhedra.points(poly)
        m = max(m, sqrNorm(x))
    end
    return sqrt(m)
end

"Compute the vertex indices of the convex hull."
function getConvexHull(v::Vector{V}) where {V<:AbstractVector{T}} where {T<:Real}
    D = length(eltype(v))
    return Int64.(MiniQhull.delaunay(stack(v), "qhull QJ"))[1:D,:]
end

"Compute the vertex indices of the Delaunay triangulation."
function getDelaunay(v::Vector{V}) where {V<:AbstractVector{T}} where {T<:Real}
    return Int64.(MiniQhull.delaunay( stack(v) ))
end

"Reflect at the unit sphere."
function polarDual(normals::Vector{V}, distances::Vector{S}) where {V<:AbstractVector{T}, S<:Real} where {T<:Real}
    D = length(normals[1])
    return [StaticArrays.MVector{D, Float64}(normals[i] ./ distances[i]) for i in eachindex(normals)]
end

"Determine the active constrains."
function getActiveConstraints(normals::Vector{V}, distances::Vector{S}) where {V<:AbstractVector{T}, S<:Real} where {T<:Real}
    ch = getConvexHull( polarDual(normals, distances) )
    active = falses( length(normals) )
    active[unique(ch)] .= true
    return active
end

"Convert a halfspace representation into a 'Polyhedra.polyhedron'."
function asPolyhedron(normals::Vector{V}, distances::Vector{S}) where {V<:AbstractVector{T}, S<:Real} where {T<:Real}
    D = length(normals[1])
    return Polyhedra.polyhedron(Polyhedra.hrep([Polyhedra.HalfSpace(SVec{D}(normals[i]), distances[i]) for i in eachindex(normals)]))
end

"Calculate the 'n'-th root of a (non-negative) number."
function root(x::Real, n::Integer=2)
    if n == 1
        return x
    end
    if n == 2
        return sqrt(x)
    end
    if n == 3
        return cbrt(x)
    end
    return exp(log(x)/n)
end